////////////////////////////////////////////////////////////////////////////////
// Purpose: Package for classes for connect_env_to_dut example
// Author: Originally from UVM basic class by John Aynsley on
//         http://verificationacademy.com but modified by Greg Tumbush
//
// REVISION HISTORY:
// $Log: $
////////////////////////////////////////////////////////////////////////////////

`include "uvm_macros.svh"

package my_package;

   import uvm_pkg::*;

class env extends uvm_env;
   `uvm_component_utils(env); // Register class env

   // Declare a virtual interface. Need to register the
   // interface with the resource database so it can be used.
   virtual dut_if dut_vi; 

   function new(string name, uvm_component parent);
      super.new(name, parent);
   endfunction // new

   function void build_phase(uvm_phase phase);
      super.build_phase(phase);
      
      // Search the config db for the name and set it equal to ref value.  cntxt is 
      //   the starting search point. inst_name is an explicit instance name relative 
      //   to cntxt and may be an empty string if the cntxt is the instance that the 
      //   configuration object applies to. 
      // Call member function get with the following fields:
      //   uvm_component cntxt: The context to find the resource in the db, similar to scope (this)
      //   string inst_name:Path to the name ("")
      //   string name: A field name to search the resource db for ("dut_vi")
      //   ref T value: Return value (dut_vi)
      // uvm_config_db:: get is a convenience function that makes use of uvm_resource_db
      uvm_config_db #(virtual dut_if)::get(this, "", "dut_vi", dut_vi);
   endfunction // build_phase

   task run_phase(uvm_phase phase);
      phase.raise_objection(this);
      dut_vi.reset = 1;
      dut_vi.data = 0;
      dut_vi.clock = 0;
      #10ns;
      dut_vi.reset = 1; // Should get: "10 Resetting
      #10ns;
      dut_vi.reset = 0;
      dut_vi.data = 8'hA5;
      #10ns;
      dut_vi.clock = 1; // Should get: "30 dut_if.data = 0xA5
      #10ns;
      dut_vi.clock = 0;
      #10ns;
      phase.drop_objection(this);
   endtask // run_phase
   
endclass // env
   
class test extends uvm_test;
   `uvm_component_utils(test); // Register class test

   virtual dut_if dut_vi;

   env env_h;




   function new(string name, uvm_component parent);
      super.new(name, parent);
   endfunction // new

   function void build_phase(uvm_phase phase);
      super.build_phase(phase);
      
      // Read name/value pair from the resource database. Since uvm_resource_db 
      //   is a parameterizable class pass in the type (virtual dut_if). This is the type
      //   of the value that is read from the resource database. It should match the type
      //   of val.
      // Call member function read_by_name with the following fields:
      //   input string scope: A scope to find the resource in the db ("dut_ifs")
      //   input string name: A field name to find the resource in the db ("dut_vi")
      //   ref (type) value: Return value through value. (dut_vi)
      // The value and name create a name/value pair. A scope is used to avoid name 
      //   collisions so you can have the same name as long as the scope is different.
      if (!uvm_resource_db #(virtual dut_if)::read_by_name("dut_ifs", "dut_vi", dut_vi))
          `uvm_fatal("test", "Cannot find resource name dut_ifs.dut_vi");

      // Create a new configuration setting for field_name and assign it to value.
      //   The setting is made at cntxt, with the full name of cntxt added to the inst_name. 
      //   Field_name is the target field. 
      // Since uvm_config_db is a parameterizable class pass in the type (virtual dut_if). This is the type
      //   of the value that is set in the config db. It should match the type of value.
      // Call member function set with the following fields:
      //   uvm_component cntx: The context to find the resource in the db, similar to scope (this)
      //   string inst_name: Path to the name, again similar to scope (*)
      //   string name: A field name to set the resource in the db ("dut_vi")
      //   value: Set name to value. (dut_vi)
      // uvm_config_db::set is a convenience function that makes use of uvm_resource_db
      uvm_config_db #(virtual dut_if)::set(this, "*", "dut_vi", dut_vi);

      env_h = env::type_id::create("env_h", this); // Creates object
   endfunction // build_phase
endclass // test1
   
   
endpackage // my_package
   
