////////////////////////////////////////////////////////////////////////////////
// Purpose: Top level module for connect_env_to_dut example
// Author: Originally from UVM basic class by John Aynsley on
//         http://verificationacademy.com but modified by Greg Tumbush
//
// REVISION HISTORY:
// $Log: $
////////////////////////////////////////////////////////////////////////////////


module top;

   import uvm_pkg::*;
   import my_package::*;

   // Instantiate I/F
   dut_if dut_bus();
   
   // Instantiate dut
   dut dut(.dut_bus(dut_bus));

   // Run the test
   initial begin
      // Store reference to interface in the resource database. Since uvm_resource_db 
      //   is a parameterizable class pass in the type (virtual dut_if). This is the type
      //   of the value that we write to the resource database. It should match the type
      //   of value.
      // Call member function set with the following fields:
      //   input string scope: A scope to find the resource in the db ("dut_ifs")
      //   input string name: A field name to find the resource in the db ("dut_vi")
      //   (type) value: Set name to value. (dut_if dut_bus)
      // The value and name create a name/value pair.
      // A scope is used to avoid name collisions so you can have the same
      //   name as long as the scope is different.
      uvm_resource_db #(virtual dut_if)::set("dut_ifs", "dut_vi", dut_bus);
      run_test("test");
   end
   
   
endmodule // top
